package implement

import (
	"code.fnuoos.com/EggPlanet/egg_models.git/src/dao"
	"code.fnuoos.com/EggPlanet/egg_models.git/src/model"
	zhios_order_relate_logx "code.fnuoos.com/EggPlanet/egg_models.git/utils/logx"
	"errors"
	"xorm.io/xorm"
)

func NewUserLevelDb(engine *xorm.Engine) dao.UserLevelDao {
	return &UserLevelDb{Db: engine}
}

type UserLevelDb struct {
	Db *xorm.Engine
}

func (u UserLevelDb) UserLevelInIDescByWeightLowWithOne() (*model.UserLevel, error) {
	var ms model.UserLevel
	has, err := u.Db.Where("is_use=1").Asc("level_weight").Get(&ms)
	if err != nil {
		return nil, err
	}
	if has == false {
		return nil, zhios_order_relate_logx.Warn(errors.New("等级不存在"))
	}
	return &ms, nil
}

func (u UserLevelDb) UserLevelByID(id interface{}) (*model.UserLevel, error) {
	m := new(model.UserLevel)
	has, err := u.Db.Where("id = ?", id).Get(m)
	if err != nil {
		return nil, err
	}
	if !has {
		return nil, errors.New("no found")
	}

	return m, nil
}

func (u UserLevelDb) UserLevelAllByAsc() ([]*model.UserLevel, error) {
	var m []*model.UserLevel
	err := u.Db.Where("is_use = ?", 1).Asc("level_weight").Find(&m)
	if err != nil {
		return nil, zhios_order_relate_logx.Warn(err)
	}
	return m, nil
}

func (u UserLevelDb) UserLevelAllByAscByPage(page, limit int) ([]*model.UserLevel, int64, error) {
	var m []*model.UserLevel
	total, err := u.Db.Where("is_use = ?", 1).Asc("level_weight").FindAndCount(&m)
	if err != nil {
		return nil, 0, zhios_order_relate_logx.Warn(err)
	}
	return m, total, nil
}

func (u UserLevelDb) UserLevelUpdate(id interface{}, userLevel *model.UserLevel, forceColumns ...string) (int64, error) {
	var (
		affected int64
		err      error
	)
	if forceColumns != nil {
		affected, err = u.Db.Where("id=?", id).MustCols(forceColumns...).Update(userLevel)
	} else {
		affected, err = u.Db.Where("id=?", id).Update(userLevel)
	}
	if err != nil {
		return 0, zhios_order_relate_logx.Error(err.Error())
	}
	return affected, nil
}

func (u UserLevelDb) UserLevelDeleteBySession(session *xorm.Session, id interface{}) (int64, error) {
	affected, err := session.Where("id = ?", id).Delete(&model.UserLevel{})
	if err != nil {
		return 0, err
	}
	return affected, nil
}

func (u UserLevelDb) UserLevelInsert(userLevel *model.UserLevel) (int, error) {
	_, err := u.Db.InsertOne(userLevel)
	if err != nil {
		return 0, zhios_order_relate_logx.Error(err)
	}
	return userLevel.Id, nil
}
